#include "IT8951_USB.h"
#include <stdio.h>
#include <math.h>

unsigned char sense_buffer[SENSE_LEN];
unsigned char data_buffer[BLOCK_LEN*256];


#define IT8951_MAX_NUM 4//2	//for 31.2/42inch
#define SCREEN_WIDTH 1440
#define SCREEN_HEIGHT 2560
DWord gulPanelW, gulPanelH;

typedef struct _DISPLAY_IMAGE_ID {
	DWord x;
	DWord y;
	DWord width;
	DWord height;
	DWord flag_needUpdate;	//0-will not update the screen, 1-will update
	Byte* bitmap;
} DisplayImage;


char *path[IT8951_MAX_NUM] = {
	"/dev/sg1",
	"/dev/sg2",
	"/dev/sg3",
	"/dev/sg0"
};

/*
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-------------------------------API START--------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
*/
//Global Virables
int evpd, page_code;
SystemInfo *Sys_info[IT8951_MAX_NUM];	// SystemInfo structure
int power_status = 0;	//1-on, 0-off


//Global APIs

/*
 * IT8951 Initialize
 */
void API_IT8951_Init() {
	evpd = page_code = 0;

	//IT8951_Cmd_AutoReset();	//DO NOT Call this Function! unless you can re-enumerate the usb devices.
	//sleep(1);

	// 0x12: get device name
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		IT8951_Cmd_Inquiry(i);
	}
	sleep(1);

	// 0x80: get system info
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		Sys_info[i]=(SystemInfo*)malloc(sizeof(SystemInfo));
	}
	IT8951_Cmd_SysInfo(Sys_info);
}
/*
 * IT8951 Deinitialize
 */
void API_IT8951_Deinit() {
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		free(Sys_info[i]);
	}
}

/*
 * Power Control
 * Params:
 *  stat 0-off, 1-on
 */
void API_IT8951_PowerCtrl(int stat) {
	if(stat == power_status) {
		printf("Power status not Changed: %d\n", stat);
		return;	//power status not changed
	}

	if(stat == 1) {
		printf("Power On VDD VGH VGL VSH VSL VCOM!\n");
		IT8951_Cmd_SetPmic(0, 1710, 1, 1, 1);	//power on and set Vcom to -1.70V
		power_status = 1;
	} else if(stat == 0) {
		printf("Power Off!\n");
		IT8951_Cmd_SetPmic(0, 0, 1, 1, 0); //power off and set Vcom off
		power_status = 0;
	} else {
		printf("Wrong Arg!\n");
	}
}

/*
 * Update Temperature
 * read temperature out from all it8951(s), calculate average temp, then write back to every it8951
 */
void API_IT8951_UpdateTemp(void) {
	TempArg TempTest;
	int value = 0;
	int avg_value;
	int i;

	TempTest.ucSetTemp = 0;		// 0: read, 1: write
	TempTest.ucTempVal = 0;	
	printf("Temperature Read: ");
	for(i=0; i<IT8951_MAX_NUM; i++){
		IT8951_Cmd_SetTemp(i, &TempTest);
		value += TempTest.ucTempVal;
		printf("%d ", TempTest.ucTempVal);
	}
	printf("\n");

	avg_value = value / IT8951_MAX_NUM;
	printf("Setting Average Temperture: %d\n", avg_value);
	for(i=0; i<IT8951_MAX_NUM; i++){
		TempTest.ucSetTemp = 1;	// 0: read, 1: write
		TempTest.ucTempVal = avg_value; 	// set value if wanna write
		IT8951_Cmd_SetTemp(i, &TempTest);
	}
}

/*
 * Load Image to IT8951 for each Zone
 * Params:
 * id: 0,1,2,3 id of it8951
 * x,y,width,height: location & size of bitmaps, 640x1440
 * src: bitmap 
 */
void API_IT8951_LoadImage_ID(int id, int x, int y, int width, int height, Byte *src) {
	IT8951_Cmd_LoadImageArea(id, src, (Sys_info[id]->uiImageBufBase), x, y, width, height);
}

/*
 * Trigger display for each Zone
 * Params:
 * id: 0,1,2,3 id of it8951
 * x,y,width,height: location & size of bitmaps, 640x1440
 * lut_mode: 0/INIT,1/DU,2/GC16 for Signage use. waveform mode
 */ 
void API_IT8951_Display_ID(int id, int x, int y, int width, int height, int lut_mode) {
	printf("%s : %d\n", __func__, id);
	IT8951_Cmd_DisplayArea(id, x, y, width, height, lut_mode, (Sys_info[id]->uiImageBufBase), 0);
}

void API_IT8951_WaitMultiEngineFree(void) {
	DWord regVal, regValMusked;
	do {
		regValMusked = 0;
		for(int i=0; i<IT8951_MAX_NUM; i++) {
			regVal = 0;
			IT8951_Cmd_RegRead(i, 0x18001224, &regVal);
			regValMusked |= regVal;
		}
	}while((regValMusked & 0xFFFF) > 0);
	usleep(30000);	//delay 30ms
}

/* 
 * Display API
 * Params:
 * x, y, width, height: size & location of bitmap on screen 2560x1440
 * lut_mode: 0/INIT,1/DU,2/GC16 for Signage use. waveform mode
 * src: bitmap /1 byte per pixel, x scan start from left-top
 */
void API_IT8951_Display(int x, int y, int width, int height, int lut_mode, Byte *src) {
	DisplayImage sub_scr[IT8951_MAX_NUM];	//4 it8951(s) means 4 sub screen
	DWord sub_id =0;

	printf("%s: %d,%d,%d,%d LutMode=%d\n", __func__, x, y, width, height, lut_mode);

	memset(sub_scr, 0x00, sizeof(DisplayImage) * IT8951_MAX_NUM);

	if((x + width > SCREEN_WIDTH) || (y + height > SCREEN_HEIGHT)) {
		printf("Picture out of range!\n");
		return;
	}

	//find which sub screen to start display
	do {
		if(y >= gulPanelH){
			sub_scr[sub_id].flag_needUpdate = 0;//picture will not be displayed on this sub screen;
			y -= gulPanelH;
			sub_id ++;
			if(y < gulPanelH) {
				break;
			} else {
				continue;
			}
		} else {
			break;
		}
	} while(1);
	printf("Picture starts at sub screen:%d\n", sub_id);

	int y0 = 0, height0 = 0;	//position on source bitmap. indicated copyed bytes;
	do{
		//not last part
		if(y + height > gulPanelH) {
			height0 = gulPanelH - y;
		// last part
		} else {
			height0 = height;
		}

		if(height0 <= 0) {
			break;
		}

		sub_scr[sub_id].flag_needUpdate = 1;
		sub_scr[sub_id].x = x;
		sub_scr[sub_id].y = y;
		sub_scr[sub_id].width = width;
		sub_scr[sub_id].height = height0;
		sub_scr[sub_id].bitmap = malloc(width * height0);
		printf("Sub ID: %d\n", sub_id);
		printf("Copy from (%d,%d,%d,%d)\n", 0, y0, width, height0);
		printf("Display to (%d,%d,%d,%d)\n",sub_scr[sub_id].x, sub_scr[sub_id].y, sub_scr[sub_id].width, sub_scr[sub_id].height);
		//copy bitmap data(0, y0, width, height0), will be displayed to (x, y, width, height0) on sub_scr[sub_id]
		for(int i=0; i<height0; i++) {
			memcpy(&(sub_scr[sub_id].bitmap[width * i]), &src[(y0 +i)* width], width);
		}
		printf("Copy Complete!\n\n\n");

		sub_id ++;
		if(sub_id > 3)
			break;

		//set params for next loop
		y = 0; //next sub screen, x start from 0
		y0 += height0;	//skip already copyed rows for next loop;
		height -= height0; //remaining rows
	}while (1);

	//divide bitmap for the rest sub screens
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		if(sub_scr[i].flag_needUpdate == 0)
			continue;
		printf("Sub ID: %d\n", i);
		printf("(%d,%d,%d,%d)\n",sub_scr[i].x, sub_scr[i].y, sub_scr[i].width, sub_scr[i].height);
	}	

#if 1
	//Load Image by ID
	DWord regVal;
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		if(sub_scr[i].flag_needUpdate == 0) {
			regVal = 0x80000;
			IT8951_Cmd_RegWrite(i, 0x18001138, &regVal);
		} else {
			API_IT8951_LoadImage_ID(i, sub_scr[i].x, sub_scr[i].y, sub_scr[i].width, sub_scr[i].height, sub_scr[i].bitmap);
		}
	}

	//Power on
	API_IT8951_PowerCtrl(1);

	//Display
	for(int i=IT8951_MAX_NUM-1; i>=0; i--) {
		if(sub_scr[i].flag_needUpdate == 0) {
		//	API_IT8951_Display_ID(i, 0,0,0,0, lut_mode);
		} else {
			API_IT8951_Display_ID(i, sub_scr[i].x, sub_scr[i].y, sub_scr[i].width, sub_scr[i].height, lut_mode);
			//API_IT8951_Display_ID(i, 0, 0, gulPanelW, gulPanelH, lut_mode);
		}
	}

	//Wait MultiEngine Free
	API_IT8951_WaitMultiEngineFree();

	//reset Reg 0x18001138
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		if(sub_scr[i].flag_needUpdate == 0) {
			regVal = 0;
			IT8951_Cmd_RegWrite(i, 0x18001138, &regVal);
		}
	}

	//Power off
	API_IT8951_PowerCtrl(0);
#endif

	//Final step;
	for(int i=0; i<IT8951_MAX_NUM; i++) {
		if(sub_scr[i].flag_needUpdate == 1) {
			free(sub_scr[i].bitmap);
			sub_scr[i].flag_needUpdate = 0;
		}
	}	
}

/*
 * IT8951 Initialize Screen. Refresh EPD with INIT mode, and reset Image Buffer to 0xF0
 */
void API_IT8951_InitScreen() {
	Byte* bitmap = malloc(gulPanelW * gulPanelH);
	//WHITE
	memset(bitmap, 0xF0, gulPanelW * gulPanelH);

	IT8951_Cmd_LoadImageArea(3, bitmap, (Sys_info[3]->uiImageBufBase), 0, 0, gulPanelW, gulPanelH);
	IT8951_Cmd_LoadImageArea(2, bitmap, (Sys_info[2]->uiImageBufBase), 0, 0, gulPanelW, gulPanelH);
	IT8951_Cmd_LoadImageArea(1, bitmap, (Sys_info[1]->uiImageBufBase), 0, 0, gulPanelW, gulPanelH);
	IT8951_Cmd_LoadImageArea(0, bitmap, (Sys_info[0]->uiImageBufBase), 0, 0, gulPanelW, gulPanelH);
	API_IT8951_PowerCtrl(1);
	IT8951_Cmd_DisplayArea(3, 0, 0, gulPanelW, gulPanelH, 0, (Sys_info[3]->uiImageBufBase), 0);
	IT8951_Cmd_DisplayArea(2, 0, 0, gulPanelW, gulPanelH, 0, (Sys_info[2]->uiImageBufBase), 0);
	IT8951_Cmd_DisplayArea(1, 0, 0, gulPanelW, gulPanelH, 0, (Sys_info[1]->uiImageBufBase), 0);
	IT8951_Cmd_DisplayArea(0, 0, 0, gulPanelW, gulPanelH, 0, (Sys_info[0]->uiImageBufBase), 0);
	API_IT8951_WaitMultiEngineFree();
	API_IT8951_PowerCtrl(0);
	
	free(bitmap);
}


int main(int argc, char * argv[]) {
	Byte* bitmap = malloc(SCREEN_WIDTH * SCREEN_HEIGHT);

	//WHITE
	memset(bitmap, 0xF0, SCREEN_WIDTH * SCREEN_HEIGHT);
	
	API_IT8951_Init();

	//Full update to white at the beginning------------------->
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_InitScreen();

	//Full/Partial Update Demo
	//Black with GC
	memset(bitmap, 0x00, SCREEN_WIDTH * SCREEN_HEIGHT);
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(40, 120, 600, 720, 2, bitmap);

	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(340, 920, 600, 720, 2, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(840, 1120, 500, 720, 2, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(40, 1600, 600, 720, 2, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(840, 20, 500, 720, 2, bitmap);

	//White with DU
	memset(bitmap, 0xF0, SCREEN_WIDTH * SCREEN_HEIGHT);
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(40, 120, 600, 720, 1, bitmap);

	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(340, 920, 600, 720, 1, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(840, 1120, 500, 720, 1, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(40, 1600, 600, 720, 1, bitmap);
	
	API_IT8951_UpdateTemp();	//Don't forget!
	API_IT8951_Display(840, 20, 500, 720, 1, bitmap);

	API_IT8951_Deinit();
	free(bitmap);
	return EXIT_SUCCESS;
}

void show_sense_buffer(struct sg_io_hdr * hdr) {
	unsigned char * buffer = hdr->sbp;

	for (int i=0; i<hdr->mx_sb_len; ++i) {
		putchar(buffer[i]);
	}
}

void IT8951_Cmd_Inquiry(int id) {

	struct sg_io_hdr * p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, data_buffer, BLOCK_LEN*256);
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		printf("\nCommand: Inquiry\n");

		status = IT8951_CMD_INQ(fd, page_code, evpd, p_hdr);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			#if defined(_TEST_SW_)
			for(int i=8; i<36; i++){
			  printf("%c", data_buffer[i]);
			}
			printf("\n");
			#endif
		}
	}
	else {
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_SysInfo(SystemInfo** Sys_info) {
	for(int id=0; id< IT8951_MAX_NUM; id++) {
		SystemInfo * temp = Sys_info[id];
		SG_IO_HDR *p_hdr = init_io_hdr();

		set_xfer_data(p_hdr, Sys_info[id], sizeof(SystemInfo));
		//printf("Xfer Length: %d\n", sizeof(SystemInfo)); 	//should be 112 bytes
		set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

		int status = 0;
		int fd = open(path[id], O_RDWR);

		if (fd>0) {
			printf("\nCommand: Get device information\n");
	
			status = IT8951_CMD_SYSTEM_INFO(fd, page_code, evpd, p_hdr);
			if (status!=0) {
				show_sense_buffer(p_hdr);
			}
			else{
				//SystemInfo *temp = p_hdr->dxferp;

				Sys_info[id]->uiStandardCmdNo = SWAP_32(temp->uiStandardCmdNo);
				Sys_info[id]->uiExtendCmdNo = SWAP_32(temp->uiExtendCmdNo);
				Sys_info[id]->uiSignature = SWAP_32(temp->uiSignature);
				Sys_info[id]->uiVersion = SWAP_32(temp->uiVersion);
				Sys_info[id]->uiWidth = SWAP_32(temp->uiWidth);
				Sys_info[id]->uiHeight = SWAP_32(temp->uiHeight);
				Sys_info[id]->uiUpdateBufBase = SWAP_32(temp->uiUpdateBufBase);
				Sys_info[id]->uiImageBufBase = SWAP_32(temp->uiImageBufBase);
				Sys_info[id]->uiTemperatureNo = SWAP_32(temp->uiTemperatureNo);
				Sys_info[id]->uiModeNo = SWAP_32(temp->uiModeNo);
	
				for(int i=0; i<8; i++){
					Sys_info[id]->uiFrameCount[i] = SWAP_32(temp->uiFrameCount[i]);
				}
	
				Sys_info[id]->uiNumImgBuf = SWAP_32(temp->uiNumImgBuf);
				printf("Number of Image Buffer: %d\n", Sys_info[id]->uiNumImgBuf);
	
				Sys_info[id]->uiWbfSFIAddr = SWAP_32(temp->uiWbfSFIAddr);

				Sys_info[id]->uiWaveformInfo = SWAP_32(temp->uiWaveformInfo);
				Sys_info[id]->uiMultiPanelIndex = SWAP_32(temp->uiMultiPanelIndex);
				Sys_info[id]->uiWbfRunType = SWAP_32(temp->uiWbfRunType);
				Sys_info[id]->uiWbfFPLotNumber = SWAP_32(temp->uiWbfFPLotNumber);
				for(int i=0; i<3; i++){
					Sys_info[id]->uiReserved[i] = SWAP_32(temp->uiReserved[i]);
				}

				gulPanelW = Sys_info[id]->uiWidth;
				gulPanelH = Sys_info[id]->uiHeight;

				printf("Panel ID: [%d]\n", Sys_info[id]->uiMultiPanelIndex);
				printf("Waveform Info: %d\n", Sys_info[id]->uiWaveformInfo);
				printf("WBF FPL ot Number: %d\n", Sys_info[id]->uiWbfFPLotNumber);
				printf("WBF SFI address: 0x%08X\n", Sys_info[id]->uiWbfSFIAddr);
				printf("Panel Width: %d\n", Sys_info[id]->uiWidth);
				printf("Panel Height: %d\n", Sys_info[id]->uiHeight);
				printf("Image buffer address: 0x%08X\n", Sys_info[id]->uiImageBufBase);
				printf("Waveform:(%d Modes, %d Temperature Segments)\n", Sys_info[id]->uiModeNo, Sys_info[id]->uiTemperatureNo);
				
			}
		}
		else{
			printf("failed to open sg file %s\n", path[id]);
		}

		close(fd);
		destroy_io_hdr(p_hdr);
	}
}

void IT8951_Cmd_MemRead(int id, DWord memAddr, Word length, Byte* revBuf) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, revBuf, length);
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		printf("\nCommand: Read memory\n");

		// Notice that the memAddr should be 0x format
		status = IT8951_CMD_READ_MEM(fd, page_code, evpd, p_hdr, memAddr, length);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			#if defined(_Test_SW_)
			printf("Read memory 0x%X: \r\n", memAddr);
			for(int i=0; i<length; i++){
				printf("%X \r", revBuf[i]);
			}
			printf("\n");
			#endif
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_MemWrite(int id, DWord memAddr, Word length, Byte* srcBuf) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, srcBuf, length);
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		status = IT8951_CMD_WRITE_MEM(fd, page_code, evpd, p_hdr, memAddr, length);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			#if defined(_TEST_SW_)
			printf("Write memory 0x%08X: 0x%02X\n", memAddr, srcBuf[0]);
			for(int i=0; i<length; i++){
				printf("%X \r", srcBuf[i]);
			}
			printf("\n");
			#endif
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_RegRead(int id, DWord regAddr, DWord *rev) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, rev, sizeof(DWord));
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		status = IT8951_CMD_READ_REG(fd, page_code, evpd, p_hdr, regAddr);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			*rev = SWAP_32(*rev);

			#if defined(_Test_SW_)
			printf("Read register 0x%X: \n", regAddr);
			printf("0x%08X \n", *rev);
			#endif
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_RegWrite(int id, DWord regAddr, DWord *src) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	DWord srcIn = SWAP_32(*src);

	set_xfer_data(p_hdr, &srcIn, sizeof(DWord));
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		status = IT8951_CMD_WRITE_REG(fd, page_code, evpd, p_hdr, regAddr);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			#if defined(_TEST_SW_)
			printf("Write register 0x%X: \r\n", regAddr);
			#endif
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_DisplayArea(int id, DWord dpyX, DWord dpyY, DWord dpyW, DWord dpyH, DWord dpyMode, DWord memAddr, DWord enWaitRdy){

	SG_IO_HDR *p_hdr = init_io_hdr();

	DisplayArg displayArg;

	displayArg.iPosX          = SWAP_32(dpyX); //Conver Little to Big for IT8951/61
    displayArg.iPosY          = SWAP_32(dpyY);
    displayArg.iWidth         = SWAP_32(dpyW);
    displayArg.iHeight        = SWAP_32(dpyH);
    displayArg.iEngineIndex   = SWAP_32(enWaitRdy);
    displayArg.iMemAddr       = SWAP_32(memAddr);
    displayArg.iWavMode       = SWAP_32(dpyMode);

	set_xfer_data(p_hdr, &displayArg, sizeof(DisplayArg));
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		printf("\nCommand: Display\n");

		status = IT8951_CMD_DPY_AREA(fd, page_code, evpd, p_hdr);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			printf("Refreshing (%d, %d, %d, %d) with mode %d ... \r\n", dpyX, dpyY, dpyW, dpyH, dpyMode);
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_LoadImageArea(int id, Byte *srcImg, DWord memAddr, DWord ldX, DWord ldY, DWord ldW, DWord ldH){

	printf("\nCommand: Load image\n");

	if(ldW <= 2048 && ldW != gulPanelW){
	#if 0		
		SG_IO_HDR *p_hdr = init_io_hdr();

		//Using IT8951 New USB Command for Loading Image - it Needs IT8951 F/W support
		DWord counter = (SPT_BUF_SIZE/ldW);
		LoadArg loadArg;

		int status = 0;
		int fd = open(path[id], O_RDWR);

		for(DWord i=0; i<ldH; i+=counter){
			if(counter > (ldH-i)){
				counter = (ldH-i);
			}
			printf("counter = %d\n", counter);

			Byte* tempBuf = (Byte*)malloc((sizeof(LoadArg)+(ldW*counter)));

			loadArg.iX = (ldX);
			loadArg.iY = ((ldY + i));
			loadArg.iW = SWAP_32(ldW);
			loadArg.iH = SWAP_32(counter);

			printf("Loading Image, %d,%d, %d*%d  at 0x%08X\n", ldX, ldY+i, ldW, counter, memAddr);
			//Set Image Buffer Start Address
			loadArg.iAddress = SWAP_32(memAddr);

			memcpy(tempBuf, &loadArg, sizeof(LoadArg));
			memcpy((tempBuf+sizeof(LoadArg)), (srcImg+(i*ldW)), (ldW*counter));

			set_xfer_data(p_hdr, tempBuf, (sizeof(LoadArg)+(ldW*counter)));
			set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

			if (fd>0) {
				status = IT8951_CMD_LOAD_IMG(fd, page_code, evpd, p_hdr);
				if (status!=0) {
					show_sense_buffer(p_hdr);
				}
				else{
					// Show some information of loaded data?
					printf("data loaded\n");
				}
			}
			else{
				printf("failed to open sg file %s\n", path[id]);
			}
			free(tempBuf);
		}

		close(fd);
		destroy_io_hdr(p_hdr);
	#else	
		//line size is always less than spt size ,so just write one line each time.
		for(DWord i=0; i<ldH; i++) {
			IT8951_Cmd_MemWrite(id, memAddr + ldX + (ldY+i) * gulPanelW,  (Word)(ldW), (srcImg + (i*ldW)));
		}	
	#endif
	}
	else{
		//Full Image - Fast Method => Send Multi Lines for each transfer
		DWord counter = (SPT_BUF_SIZE/ldW);
		printf("counter = %d\n", counter);
		for(DWord i=0; i<ldH; i+=counter){
			if(counter > (ldH-i)){
				counter = (ldH-i);
			}

			//We Send Multi Lines for each Bulk Transfer
			IT8951_Cmd_MemWrite(id, (memAddr+ldX+((ldY+i)*gulPanelW)), (Word)(ldW*counter), (srcImg+(i*ldW)));
		}
	}
}

void IT8951_Cmd_SetTemp(int id, TempArg *SetTemp) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	Byte flag = SetTemp->ucSetTemp;
	Byte value = SetTemp->ucTempVal;

	set_xfer_data(p_hdr, SetTemp, sizeof(TempArg)/sizeof(Byte));
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		printf("\nCommand: Set temperature\n");

		status = IT8951_CMD_SET_TEMP(fd, page_code, evpd, p_hdr, SetTemp->ucSetTemp, SetTemp->ucTempVal);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			if((flag)==0x00){
				TempArg *temp = (p_hdr->dxferp);
				Byte TempByte;

				// will return a word , need to swap
				TempByte = temp->ucSetTemp;
				temp->ucSetTemp = temp->ucTempVal;
				temp->ucTempVal = TempByte;
				SetTemp->ucTempVal = TempByte;

				printf("Get temperature: %X\n", temp->ucTempVal);
			}
			else{
				printf("Set temperature, now forced as %d\n", value);
				printf("Please read temperature again to check it.\n");
			}
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}


void IT8951_Cmd_SetPmic(int id, DWord Vcom, Byte doSetVcom, Byte doSetPower, Byte OnOff) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, NULL, 0);
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {
		printf("\nCommand: PMIC Control\n");

		status = IT8951_CMD_SET_PMIC(fd, page_code, evpd, p_hdr, Vcom, doSetVcom, doSetPower, OnOff);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			printf("Vcom: %d, Power: %d\n", Vcom, OnOff);
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}

void IT8951_Cmd_AutoReset(int id) {

	SG_IO_HDR *p_hdr = init_io_hdr();

	set_xfer_data(p_hdr, NULL, 0);
	set_sense_data(p_hdr, sense_buffer, SENSE_LEN);

	int status = 0;
	int fd = open(path[id], O_RDWR);

	if (fd>0) {

		status = IT8951_CMD_AUTO_RESET(fd, page_code, evpd, p_hdr);
		if (status!=0) {
			show_sense_buffer(p_hdr);
		}
		else{
			printf("AUTO RESET ... \n");
		}
	}
	else{
		printf("failed to open sg file %s\n", path[id]);
	}

	close(fd);
	destroy_io_hdr(p_hdr);
}


